/* ------------------------------------------------------------------------ */
/* --                SQL Interface Class for mySQL Access                -- */
/* --                                                                    -- */
/* --                Thomas Jungmann thj@gmx.net                         -- */
/* --                last modified 03.12.2000                            -- */
/* --                This interface uses RexxSQL by Mark Hessling        -- */
/* ------------------------------------------------------------------------ */

.local~sessionTimeOut = 20                      --Minutes after a session is closed after last user action
.local~DBConnectionIdentifier ="c1"             --Currently an arbitrary string
.local~DBUserName = "calendar"                  --MySQL-Username
.local~DBPassword = "wauwau"                    --MySQL-Password for User
.local~DBDatabase = "calendarstore"             --MySQL-Database name
.local~DBHost = "matush.wi-inf.uni-essen.de"    --FQ Hostname of MySQL-Server



::CLASS SQLInterface PUBLIC

    ::METHOD INIT
        --Constructor--------------
        /* loads the required library 'RexxSQL' automagically   */
        /* and initiates the connection to the database         */
        ---------------------------
        if rxFuncQuery("SQLLoadFuncs") then do
            call rxFuncAdd "SQLLoadFuncs","rexxsql","SQLLoadFuncs"
            call SQLLoadFuncs
        end
        self~establishDBConnection()

    ::METHOD UNINIT
        --Destructor---------------
        /* Disconnects from database and                */
        /* unloads the library 'RexxSQL'                */
        ---------------------------
        self~dropDBConnection()
        call SQLDropFuncs "UNLOAD"

    ::METHOD establishDBConnection PRIVATE
        ---------------------------
        /* need to connect to database before querys can be issued */
        ---------------------------
        res = SQLConnect(.DBConnectionIdentifier,.DBUserName,.DBPassword,.DBDatabase,.DBHost)

    ::METHOD dropDBConnection
        ---------------------------
        /* disconnects from database */
        ---------------------------
        res = SQLDisconnect(.DBConnectionIdentifier)

    ::METHOD getAllUsers PRIVATE
        ---------------------------
        /*  returns a Rexx-Stem Variable (Array) with all users.
            Can be accessed from the calling script like this:  
            a. = sqli~getAllUsers()
            do i=1 to a.0
                say a.uID.i a.uniqueName.i a.password.i
            end                                         */
        ---------------------------
        res = SQLCommand(SQLout,"select * from user;")
        return SQLout.

    ::METHOD getNewUID PRIVATE
        ---------------------------
        /*  internal Method, used for finding a new user-ID
            (like MySQLs auto_increment - function)     */
        ---------------------------
        cmd = "select MAX(uID) as mxid from user;"
        res = SQLCommand(SQLout,cmd)
        maxID = SQLout.mxid.1
        return (maxID+1)

    ::METHOD createNewUser
        ---------------------------
        /*  Inserts a new User into the database
            Arguments:  uName       = Username
                        uPassword   = Users password
            Returns:        -1      on Error while DB-Access
                            -2      bad username or password
                        UserID      when successful */
        ---------------------------
        parse arg uName,uPasswd
        if uName="-1" | uPasswd="-1" then       /* "-1" is reserved as a flag value and should not  */
            return -2                           /* be used as UserName/Password                     */
        uid = self~getNewUID()
        cmd = "insert into user values("||uid||","||d2c(39)||uName||d2c(39)||","||d2c(39)||uPasswd||d2c(39)||");"
        res = SQLCommand(SQLout,cmd)
        if res \= 0 then
            return -1
        return(uid)

    ::METHOD deleteUser
        ---------------------------
        /*  Permanently deletes User from database
            Arguments:  uID         = UserID
            Returns:        0       success
                        other       failure */
        ---------------------------
        parse arg uid
        cmd = "delete from user where uid="||uid||";"
        res = SQLCommand(SQLout,cmd)
        return res                      /* res=0 => success, else failure */

    ::METHOD getUserID
        ---------------------------
        /*  Retrieves the UserID for a given Nickname, so Users don't have
            to remember IDs but only Names.
            Arguments:  uniqueName  = Users Nickname
            Returns:    -1          on DB-Access Error
                        -2          User not found
                        UserID      when Nickname was found         */
        ---------------------------
        parse arg uniqueName
        cmd = "select uID from user where uniqueName="||d2c(39)||uniqueName||d2c(39)||";"
        res = SQLCommand(SQLout,cmd)
        if res \= 0 then
            return -1                   /* Error while DB-Access */
        if SQLout.uID.0 then            /* SQLout.uID.0 = Resultcount of SQLCommand: 1=TRUE=found */
            return SQLout.uID.1         /* SQLout.uID.1 contains the userID         */
        else
            return -2                   /* SQLCommand returned empty list => Name not found! */

    ::METHOD getUserName
        ---------------------------
        /*  Retrieves the UserName (Nickname) for a given UserID
            Arguments:  uID         = UserID
            Returns:    -1          on Error
                        userName    when uID exists                 */
        ---------------------------
        parse arg uID
        cmd = "select uniqueName from user where uID="||d2c(39)||uID||d2c(39)||";"
        res = SQLCommand(SQLout,cmd)
        if SQLout.uniqueName.0 then     /* SQLout.uniqueName.0 = Resultcount of SQLCommand: 1=TRUE=found! */
            return SQLout.uniqueName.1
        else
            return -1                   /* SQLCommand returned empty list => userID not found! */

    ::METHOD getUserPwd
        ---------------------------
        /*  Returns the password
            Arguments:  uID         = UserID
            Returns:    -1          on DB-Access Error
                        -2          when userID not found
                        password    when there is one       */
        ---------------------------
        parse arg uID
        cmd = "select password from user where uID="||d2c(39)||uID||d2c(39)||";"
        res = SQLCommand(SQLout,cmd)
        if res \= 0 then
            return -1
        if SQLout.password.0 then       /* SQLout.password.0 = Resultcount of SQLCommand: 1=TRUE=found! */
            return SQLout.password.1
        else
            return -2                   /* SQLCommand returned empty list => userID not found! */

    ::METHOD setUserPwd
        ---------------------------
        /*  Sets/Changes user password
            Arguments:  uID         = userID
                        passwd      = users password
            Returns:    <0          error
                        0           password changed        */
        ---------------------------
        parse arg uID,passwd
        if passwd = "-1" then
            return -2                           /* BAD IDEA, -1 if a flag value... */
        cmd = "update user set password="||d2c(39)||passwd||d2c(39)||" where uID="||d2c(39)||uID||d2c(39)||";"
        res = SQLCommand(SQLout,cmd)
        if res \= 0 then
            return -1                           /* some DB-Access Error */
        passwdtest = self~getUserPwd(uID)
        if passwdtest = -1 then
            return -3                           /* User does not exist */
        else
            if passwdtest \= passwd then
                return -4                       /* Password has not been changed for some unknown reason */
        return 0

    ::METHOD createSID
        ---------------------------
        /*  Creates a session ID and set the expire time n minutes into the future, where
            n is retrieved from the local environment:
            n = .local~sessionTimeOut       (defined at the beginning of this script)
            Arguments:  uID         = User ID
            Returns:    sID         = a session identifier, valid for n minutes     */
        ---------------------------
        parse arg uID
        tme = time("L")
        parse var tme "." t                 -- t = microseconds of system time (6 Digits)
        r = random(1000,9999,t)             -- r = random number between 1000 and 9999, using t as random seed
        sID = r||uID||t                     -- complete SID is a concatination of t,r and userID
        cmd = "insert into session values("||d2c(39)||sID||d2c(39)||","
        cmd = cmd||d2c(39)||uID||d2c(39)||",DATE_ADD(NOW(),INTERVAL "||.sessionTimeOut||" MINUTE));"
        res = SQLCommand(SQLout,cmd)
        if res = 0 then
            return sID
        return -1                           -- some error, don't know what, this is not supposed to happen

    ::METHOD findUID PRIVATE
        ---------------------------
        /*  Finds out who owns a specific SID but does not change the session expire (like getUID4SID)
            Arguments:  checkSID    = session ID
            Returns:    uID         = userID for user who owns the session (if still valid)  
                        -1          on error
                        -2          session not found, probably invalid */
        ---------------------------
        parse arg checkSID
        cmd = "select uID from session where (sID="
        cmd = cmd||d2c(39)||checkSID||d2c(39)||") and (expireTime > now());"
        res = SQLCommand(SQLout,cmd)
        if res \= 0 then
            return -1                           /* some error while DB-Access */
        if SQLout.uID.0 = 0 then
            return -2                           /* Session not valid (either no such sID or session timed out) */
        return SQLout.uID.1                     /* return userID if session still valid */
        
    ::METHOD getUID4SID
        ---------------------------
        /*  Finds out who owns a specific SID and prolongs the session expire
            Arguments:  checkSID    = session ID
            Returns:    uID         = userID for user who owns the session (if still valid)   */
        ---------------------------
        parse arg checkSID
        uid = self~findUID(checkSID)
        if uid < 0 then
            return uid                                  /* error, see findUID for return value details*/
        res = self~updateSessionExpire(checkSID)        /* increase session expire */
        return uid

    ::METHOD closeSession
        ---------------------------
        /*  Deletes Session from database after logout (or timeout)
            Arguments:  sessionID   = sessionID of session to delete
            Returns:    -1          on error
                        0           on success          */
        ---------------------------
        parse arg sessionID
        cmd = "delete from session where sID="||d2c(39)||sessionID||d2c(39)||";"
        res = SQLCommand(SQLout,cmd)
        if res \=0 then
            return -1
        return 0
        
    ::METHOD updateSessionExpire
        ---------------------------
        /*  When this method is invoked, the timeout for the specified session is
            postponed (.local~sessionTimeOut) minutes into the future
            Arguments:  sessionID   = session identifier of session to be prolonged
            Returns:    -1          on error
                        0           on success              */
        ---------------------------
        parse arg sessionID
        checkSession = self~findUID(sessionID)
        if checkSession < 0 then                /* Session does not exist or already expired */
            return checkSession                 /* or unable to query DB */
        cmd = "update session set expireTime=DATE_ADD(NOW(),INTERVAL "||.sessionTimeOut
        cmd = cmd||" MINUTE) where sID="||d2c(39)||sessionID||d2c(39)||";"
        res = SQLCommand(SQLout,cmd)
        if res \= 0 then
            return -1                           /* some error while DB-Access */
        return 0
        
    ::METHOD deleteAllExpiredSID
        ---------------------------
        /*  Deletes all sessions from the database that are expired by now.
            Sessions may remain in the database, when user does not log himself out properly,
            so this should be done from time to time.
            Arguments:  none at all
            Returns:    0           on success
                        else        on error                        */
        ---------------------------
        cmd = "delete from session where expireTime<now();"
        res = SQLCommand(SQLout,cmd)
        return

    ::METHOD createEvent
        ---------------------------
        /*  Insert an event (incl. optional properties) for a specific user into the database 
            Arguments:  userID          
                        categoryID              number according to the category definition table
                        eventDescription        not more than 100 characters
                        eventDate               in MySQL-Date-Format: 'yyyy-mm-dd'
                        eventTime               in MySQL-Time-Format: 'hh:mm:ss' (seconds must not be omitted!)
                        eventDuration           same Time-Format as above
            Returns:    0                       success
                        else                    error               */
        ---------------------------
        parse arg userID,categoryID,eventDescription,eventDate,eventTime,eventDuration
        cmd = "insert into event values("||copies(d2c(39),2)||","||d2c(39)||userID||d2c(39)||","
        cmd = cmd||categoryID||","||d2c(39)||eventDescription||d2c(39)||","
        cmd = cmd||d2c(39)||eventDate||d2c(39)||","||d2c(39)||eventTime||d2c(39)||","
        cmd = cmd||d2c(39)||eventDuration||d2c(39)||");"
        res = SQLCommand(SQLout,cmd)
        return res                                /* res=0 => success, else failure */

    ::METHOD deleteEvent
        ---------------------------
        /*  Delete specified event
            Arguments:  eventID         event to be deleted
            Return:     0               allright
                        other           some error          */
        ---------------------------
        parse arg eventID
        cmd = "delete from event where eventID="||d2c(39)||eventID||d2c(39)||";"
        res = SQLCommand(SQLout,cmd)
        return res                  /* res=0 => success, else failure */

    ::METHOD updateEvent
        -------------------------------------------------------------------------
        /*  Change event properties. No argument may be omitted.
            Arguments:  eventID
                        categoryID              number according to the category definition table
                        eventDescription        not more than 100 characters
                        eventDate               in MySQL-Date-Format: 'yyyy-mm-dd'
                        eventTime               in MySQL-Time-Format: 'hh:mm:ss' (seconds may not be omitted!)
                        eventDuration           same Time-Format as above
            Returns:    0                       OK
                        else                    Failure             */
        -------------------------------------------------------------------------
        parse arg eventID,categoryID,eventDescription,eventDate,eventTime,eventDuration
        cmd = "update event set categoryID="||d2c(39)||categoryID||d2c(39)||", "
        cmd = cmd||"description="||d2c(39)||eventDescription||d2c(39)||", "
        cmd = cmd||"eventDate="||d2c(39)||eventDate||d2c(39)||", "
        cmd = cmd||"eventTime="||d2c(39)||eventTime||d2c(39)||", "
        cmd = cmd||"duration="||d2c(39)||eventDuration||d2c(39)
        cmd = cmd||" where eventID="||d2c(39)||eventID||d2c(39)||";"
        res = SQLCommand(SQLout,cmd)
        return res                   /* res=0 => success, else failure */

    ::METHOD retrieveEvent
        ---------------------------
        /*  Returns all info about a specific event in a Rexx Stem Variable (Array)
            Arguments:  eventID         Unique identifier of event
            Returns:    SQLout.         Array with all columns of the event the database supplies */
        ---------------------------
        parse arg eventID
        cmd = "select * from event where eventID="||d2c(39)||eventID||d2c(39)||";"
        res = SQLCommand(SQLout,cmd)
        return SQLout.  
    
    ::METHOD findEvent
        ---------------------------
        /*  Find all events for a user that matches a specific category, could be useful someday...
            Arguments:  userID
                        categoryID      identifier of the category
                        
            Returns:    SQLout.         Array with all events of a category  */
        ---------------------------
        parse arg userID,categoryID
        cmd = "select eventID from event where categoryID="||d2c(39)||categoryID||d2c(39)
        cmd = cmd||" and uID="||d2c(39)||userID||d2c(39)||" order by eventID;"
        res = SQLCommand(SQLout,cmd)
        return SQLout.

    ::METHOD getEventsOfDay
        ---------------------------
        /*  Returns all eventIDs of a specific date and user
            Arguments:  userID
                        eventDt                     date of event
                        
            Returns:    SQLout.                     Array with all events of one day        
                        SQLout.eventID.0=0          when no event found                 */
        ---------------------------
        parse arg userID,eventDt
        cmd = "select eventID,eventTime,description from event where eventDate="||d2c(39)||eventDt||d2c(39)
        cmd = cmd||" and uID="||d2c(39)||userID||d2c(39)||" order by eventTime;"
        res = SQLCommand(SQLout,cmd)
        if SQLout.eventID.0 = 'SQLOUT.EVENTID.0' then SQLout.eventID.0 = 0
        return SQLout.

    ::METHOD getCategory
        ---------------------------
        /*  Returns the name of a given categoryID
            Arguments:  categoryID
                        
            Returns:    SQLout.category.1           Name of category
                        -1                          DB-Error
                        -2                          no such category          */
        ---------------------------
        parse arg catID
        cmd = "select category from category where categoryID="||catID||";"
        res = SQLCommand(SQLout,cmd)
        if res \=0 then return res
        if SQLout.category.0 = 0 then return -2
        return SQLout.category.1

    ::METHOD getMaxCategory
        ---------------------------
        /*  Returns the name of a given categoryID
            Arguments:  none
                        
            Returns:    SQLout.category.1           max. Indexnumber of category
                        -1                          DB-Error                    */
        ---------------------------
        cmd = "select max(categoryID) as mxid from category;"
        res = SQLCommand(SQLout,cmd)
        if res \=0 then return res
        return SQLout.mxid.1
        